#!/bin/sh
#
# Japan internet radio live streaming recoder
# Copyright (C) 2019 uru (https://twitter.com/uru_2)
# License is MIT (see LICENSE file)
set -u

#######################################
# Show usage
# Arguments:
#   None
# Returns:
#   None
#######################################
show_usage() {
  cat << _EOT_
Usage: $(basename "$0") [options]
Options:
  -s STATION ID   Station ID
  -i ADDRESS      login mail address (radiko only)
  -p PASSWORD     login password (radiko only)
  -l              Show all station ID list
_EOT_
}

#######################################
# Show all station ID and name
# Arguments:
#   None
# Returns:
#   None
#######################################
show_all_stations() {
  # radiko
  echo "Record type: radiko"
  list=$(curl --silent "https://radiko.jp/v3/station/region/full.xml")
  cnt=$(echo "${list}" | xmllint --xpath "count(/region/stations/station)" - 2> /dev/null)
  for i in $(awk "BEGIN { for (i = 1; i <= ${cnt}; i++) { print i } }"); do
    echo "  $(echo "${list}" | xmllint --xpath "concat((/region/stations/station)[${i}]/id/text(), ': ', (/region/stations/station)[${i}]/name/text())" - 2> /dev/null)"
  done
  echo ""
}

#######################################
# Radiko Login
# Arguments:
#   Mail address
#   Password
# Returns:
#   0: Success
#   1: Failed
#######################################
login_radiko() {
  mail=$1
  password=$2

  # Login
  login_json=$(curl \
      --silent \
      --request POST \
      --data-urlencode "mail=${mail}" \
      --data-urlencode "pass=${password}" \
      --output - \
      "https://radiko.jp/v4/api/member/login")

  # Extract login result
  radiko_session=$(echo "${login_json}" | jq -r ".radiko_session")
  areafree=$(echo "${login_json}" | jq -r ".areafree")

  # Check login
  if [ -z "${radiko_session}" ] || [ "${areafree}" != "1" ]; then
    return 1
  fi

  echo "${radiko_session}"
  return 0
}

#######################################
# Radiko Logout
# Arguments:
#   radiko Session
# Returns:
#   None
#######################################
logout_radiko() {
  radiko_session=$1

  # Logout
  curl \
    --silent \
    --request POST \
    --data-urlencode "radiko_session=${radiko_session}" \
    --output /dev/null \
    "https://radiko.jp/v4/api/member/logout"
}

#######################################
# Authorize radiko
# Arguments:
#   radiko Session
# Returns:
#   0: Success
#   1: Failed
#######################################
radiko_authorize() {
  radiko_session=$1

  # Define authorize key value (from https://radiko.jp/apps/js/playerCommon.js)
  RADIKO_AUTHKEY_VALUE="bcd151073c03b352e1ef2fd66c32209da9ca0afa"

  # Authorize 1
  auth1_res=$(curl \
      --silent \
      --header "X-Radiko-App: pc_html5" \
      --header "X-Radiko-App-Version: 0.0.1" \
      --header "X-Radiko-Device: pc" \
      --header "X-Radiko-User: dummy_user" \
      --dump-header - \
      --output /dev/null \
      "https://radiko.jp/v2/api/auth1")

  # Get partial key
  authtoken=$(echo "${auth1_res}" | awk 'tolower($0) ~/^x-radiko-authtoken: / {print substr($0,21,length($0)-21)}')
  keyoffset=$(echo "${auth1_res}" | awk 'tolower($0) ~/^x-radiko-keyoffset: / {print substr($0,21,length($0)-21)}')
  keylength=$(echo "${auth1_res}" | awk 'tolower($0) ~/^x-radiko-keylength: / {print substr($0,21,length($0)-21)}')
  if [ -z "${authtoken}" ] || [ -z "${keyoffset}" ] || [ -z "${keylength}" ]; then
    return 1
  fi
  partialkey=$(echo "${RADIKO_AUTHKEY_VALUE}" | dd bs=1 "skip=${keyoffset}" "count=${keylength}" 2> /dev/null | base64)

  # Authorize 2
  auth2_url_param=""
  if [ -n "${radiko_session}" ]; then
    auth2_url_param="?radiko_session=${radiko_session}"
  fi
  curl \
      --silent \
      --header "X-Radiko-Device: pc" \
      --header "X-Radiko-User: dummy_user" \
      --header "X-Radiko-AuthToken: ${authtoken}" \
      --header "X-Radiko-PartialKey: ${partialkey}" \
      --output /dev/null \
      "https://radiko.jp/v2/api/auth2${auth2_url_param}"
  ret=$?
  if [ ${ret} -ne 0 ]; then
    return 1
  fi

  echo "${authtoken}"
  return 0
}

#######################################
# Get radiko HLS streaming URI
# Arguments:
#   Station ID
#   radiko login status
# Returns:
#   None
#######################################
get_hls_uri_radiko() {
  station_id=$1
  radiko_login_status=$2

  areafree="0"
  if [ "${radiko_login_status}" = "1" ]; then
    areafree="1"
  fi

  curl --silent "https://radiko.jp/v2/station/stream_smh_multi/${station_id}.xml" | xmllint --xpath "/urls/url[@areafree='${areafree}'][1]/playlist_create_url/text()" - 2> /dev/null
}

#######################################
# Format time text
# Arguments:
#   Time minute
# Returns:
#   None
#######################################
format_time() {
  minute=$1

  hour=$((minute / 60))
  minute=$((minute % 60))

  printf "%02d:%02d:%02d" "${hour}" "${minute}" "0"
}


##### Main routine start #####

# Argument none?
if [ $# -lt 1 ]; then
  exit 1
fi

# Parse argument
station_id=""
login_id=""
login_password=""
while getopts s:i:p:l option; do
  case "${option}" in
    s)
      station_id="${OPTARG}"
      ;;
    i)
      login_id="${OPTARG}"
      ;;
    p)
      login_password="${OPTARG}"
      ;;
    l)
      show_all_stations
      exit 0
      ;;
  esac
done

# Set value from ENV
if [ -z "${login_id}" ]; then
  env | grep -q -E "^RADIKO_MAIL="
  ret=$?
  if [ ${ret} -eq 0 ]; then
    login_id="${RADIKO_MAIL}"
  fi
fi
if [ -z "${login_password}" ]; then
  env | grep -q -E "^RADIKO_PASSWORD="
  ret=$?
  if [ ${ret} -eq 0 ]; then
    login_password="${RADIKO_PASSWORD}"
  fi
fi

if [ -z "${station_id}" ]; then
  # -s value is empty
  echo "Require \"Station ID\"" >&2
  exit 1
fi

playlist_uri=""
radiko_authtoken=""

# radiko
radiko_session=""
radiko_login_status="0"

# Login radiko premium
if [ -n "${login_id}" ]; then
  radiko_session=$(login_radiko "${login_id}" "${login_password}")
  ret=$?
  if [ ${ret} -ne 0 ]; then
    echo "Cannot login radiko premium" >&2
    exit 1
  fi

  # Register radiko logout handler
  trap "logout_radiko ""${radiko_session}""" EXIT HUP INT QUIT TERM

  radiko_login_status="1"
fi

# Authorize
radiko_authtoken=$(radiko_authorize "${radiko_session}")
ret=$?
if [ ${ret} -ne 0 ]; then
  echo "radiko authorize failed" >&2
  exit 1
fi

playlist_uri=$(get_hls_uri_radiko "${station_id}" "${radiko_login_status}")

if [ -z "${playlist_uri}" ]; then
  echo "Cannot get playlist URI" >&2
  exit 1
fi

# print play env
echo "login id: ${login_id}"
echo "login password: ${login_password}"
echo "station id: ${station_id}"
echo "radio uri: ${playlist_uri}"

# Play
ffplay \
  -nodisp \
  -vn \
  -loglevel error \
  -fflags +discardcorrupt \
  -headers "X-Radiko-Authtoken: ${radiko_authtoken}" \
  -i "${playlist_uri}" 2>&1 &
# Finish
exit 0
##### Main routine end #####
